﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using Infragistics.Olap.FlatData;
using Infragistics.Olap;
using System.Collections;
using System.Reflection;
using System.Text.RegularExpressions;


namespace FromDataSet
{
    public class DataHelper
    {
        public DataSet GetDataFromDB()
        {

            string connStr = "Data Source=(local);Initial Catalog=AdventureWorks;Integrated Security=True;MultipleActiveResultSets=True";
            SqlConnection conn = new SqlConnection(connStr);

            conn.Open();

            SqlDataAdapter daProducts = new SqlDataAdapter("Select * from Production.Product",conn);

            DataSet ds = new DataSet();

            daProducts.FillSchema(ds, SchemaType.Source, "Product");
            daProducts.Fill(ds, "Product");

            return ds;

        }

        public FlatDataSource CreateFlatDataSource(DataTable dt)
        {

            DynamicTypeBuilder typeBuilder = new DynamicTypeBuilder
            {
                DynamicAssemblyName = "MyAssembly",
                DynamicTypeName = "Product"
            };

            IList<DynamicTypePropertyInfo> properties = new List<DynamicTypePropertyInfo>();

            foreach (DataColumn column in dt.Columns)
            {
                string name = AddSpaces(column.ColumnName);
                //string name = column.ColumnName;
                DynamicTypePropertyInfo propertyInfo = new DynamicTypePropertyInfo
                {
                    
                    PropertyName = name,
                    PropertyType = column.DataType
                };

                properties.Add(propertyInfo);
            }

            // Add extra calculated measure
            DynamicTypePropertyInfo propertyInfo2 = new DynamicTypePropertyInfo
            {

                PropertyName = "CalcMeasure",
                PropertyType = typeof(Decimal)
            };
            properties.Add(propertyInfo2);

            Type dynamicType = typeBuilder.GenerateType(properties);

            Type listType = typeof(List<>);

            Type genericListType = listType.MakeGenericType(dynamicType);

            IList list = (IList)Activator.CreateInstance(genericListType);

            foreach (DataRow dataRow in dt.Rows)
            {

                object myDynamicInstance = Activator.CreateInstance(dynamicType);

                foreach (DataColumn column in dt.Columns)
                {
                    string name = AddSpaces(column.ColumnName);
                    //string name = column.ColumnName;
                    PropertyInfo propertyVal = dynamicType.GetProperty(name);

                    if (dataRow[column] != DBNull.Value)
                    {
                        propertyVal.SetValue(myDynamicInstance, dataRow[column], null);
                    }
                }

                // Add Calculated Measure
                PropertyInfo propertyVal2 = dynamicType.GetProperty("CalcMeasure");
                if (dataRow["ListPrice"] != DBNull.Value && dataRow["StandardCost"] != DBNull.Value &&
                    ((Decimal)dataRow["ListPrice"]) != 0)
                {
                    Decimal LP = (Decimal)dataRow["ListPrice"];
                    Decimal SC = (Decimal)dataRow["StandardCost"];
                    Decimal calVal = (LP - SC) / LP;
                    propertyVal2.SetValue(myDynamicInstance, calVal, null);
                }


                list.Add(myDynamicInstance);

            }




            FlatDataSource flatDataSource = new FlatDataSource()
            {

                ItemsSource = list,
                Cube = DataSourceBase.GenerateInitialCube("Product"),

                // if you know the names of demensions you want to be in rows and columns 

                // you can define them here

                Rows = DataSourceBase.GenerateInitialItems("[Class].[Class]"),
                Columns = DataSourceBase.GenerateInitialItems("[Product Line].[Product Line]"),
                Measures = DataSourceBase.GenerateInitialItems("Standard Cost")
              

            };
           
            CubeMetadata cmd = new CubeMetadata();
            cmd.DataTypeFullName = "Product";
            
            cmd.DisplayName = "Product Data";

            

            DimensionMetadata dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Name";
            dmd.DisplayName = "Name Test";
            cmd.DimensionSettings.Add(dmd);

            dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Product Line";
            dmd.DisplayName = "Product Line";
            cmd.DimensionSettings.Add(dmd);

          
            dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Sell end Date";
            dmd.DisplayName = "Sell End Date";
            
            cmd.DimensionSettings.Add(dmd);
             
           
            // Change Measure Name
            dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Days to Manufacture";
            dmd.DisplayName = "TimeFrame";
            dmd.DimensionType = DimensionType.Measure;
            cmd.DimensionSettings.Add(dmd);
            

            dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Color";
            dmd.DisplayName = "Color";
              
            dmd.DimensionType = DimensionType.Measure;
            dmd.Aggregator = new TopCountOfStringAggregator();
            cmd.DimensionSettings.Add(dmd);

            dmd = new DimensionMetadata();
            dmd.SourcePropertyName = "Make Flag";
            dmd.DisplayName = "Make Flag";

            dmd.DimensionType = DimensionType.Measure;
            dmd.Aggregator = new TopCountBoolAggregator();
            cmd.DimensionSettings.Add(dmd);

            HierarchyDescriptor hd = new HierarchyDescriptor();
            hd.SourcePropertyName = "Product Line";
            HierarchyLevelDescriptor hld = new HierarchyLevelDescriptor();
            hld.LevelName = "All Lines";
            hd.LevelDescriptors.Add(hld);
            hld = new HierarchyLevelDescriptor();
            hld.LevelName = "Line";
            hld.LevelExpressionPath = "Product Line";
            hd.LevelDescriptors.Add(hld);

            flatDataSource.HierarchyDescriptors.Add(hd);


            flatDataSource.CubesSettings.Add(cmd);
            //flatDataSource.Cube = DataSourceBase.GenerateInitialCube("Product");      
            //flatDataSource.Rows = DataSourceBase.GenerateInitialItems("[Name].[Name]");
            //flatDataSource.Columns = DataSourceBase.GenerateInitialItems("[Product Line].[Product Line]");
         //   flatDataSource.DimensionsGenerationMode = DimensionsGenerationMode.Metadata;
            return flatDataSource;
                        
        }

        private string AddSpaces(string origName)
        {
            Regex r = new Regex("([A-Z]+[a-z]+)"); 
            string result = r.Replace(origName, m => (m.Value.Length > 3 ? m.Value : m.Value.ToLower()) + " ");
            return result.Trim();
        }
    }
   
    

}
